//+------------------------------------------------------------------+
//|                                                  SidusSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "Sidus.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    Sidus                    |
//| Type=SignalAdvanced                                              |
//| Name=Sidus                                                       |
//| Class=CSidusSignal                                               |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=FastEMA,uint,18,  EMA                     |
//| Parameter=SlowEMA,uint,28,  EMA                   |
//| Parameter=FastLWMA,uint,5,  LWMA                    |
//| Parameter=SlowLWMA,uint,8,  LWMA                  |
//| Parameter=IPC,ENUM_APPLIED_PRICE,PRICE_CLOSE,    |
//| Parameter=Digit,uint,4,                            |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CSidusSignal.                                              |
//| :            |
//|  Sidus http://www.mql5.com/ru/code/751/.               |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CSidusSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      Sidus

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   Sidus

   uint              m_FastEMA;          //   EMA
   uint              m_SlowEMA;          //   EMA
   uint              m_FastLWMA;         //   LWMA
   uint              m_SlowLWMA;         //   LWMA
   uint              m_IPC;              //  
   uint              m_Digit;            //   
   uint              m_SignalBar;        //     

public:
                     CSidusSignal();

   //---    
   void               BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void               SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void               BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void               SellPosClose(bool value)                { m_SellPosClose=value;     }
   void               Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void               FastEMA(uint value)                     { m_FastEMA=value;          }
   void               SlowEMA(uint value)                     { m_SlowEMA=value;          }
   void               FastLWMA(uint value)                    { m_FastLWMA=value;         }
   void               SlowLWMA(uint value)                    { m_SlowLWMA=value;         }
   void               IPC(uint value)                         { m_IPC=value;              }
   void               Digit(uint value)                       { m_Digit=value;            }
   void               SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitSidus(CIndicators *indicators);      // ,   Sidus
  };
//+------------------------------------------------------------------+
//|  CSidusSignal.                                        |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CSidusSignal::CSidusSignal()
  {
//---   -
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---   
   m_Ind_Timeframe=PERIOD_H4;
   m_FastEMA=18;
   m_SlowEMA=28;
   m_FastLWMA=5;
   m_SlowLWMA=8;
   m_IPC=PRICE_CLOSE;
   m_Digit=0;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSidusSignal::ValidationSettings()
  {
//---  
   if(m_FastEMA<=0)
     {
      printf(__FUNCTION__+":   EMA    ");
      return(false);
     }

   if(m_SlowEMA<=0)
     {
      printf(__FUNCTION__+":   EMA    ");
      return(false);
     }

   if(m_FastLWMA<=0)
     {
      printf(__FUNCTION__+":   LWMA    ");
      return(false);
     }

   if(m_SlowLWMA<=0)
     {
      printf(__FUNCTION__+":   LWMA    ");
      return(false);
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSidusSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitSidus(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   Sidus.                                  |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSidusSignal::InitSidus(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[7];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="Sidus.ex5";

   parameters[1].type=TYPE_INT;
   parameters[1].integer_value=m_FastEMA;

   parameters[2].type=TYPE_INT;
   parameters[2].integer_value=m_SlowEMA;

   parameters[3].type=TYPE_INT;
   parameters[3].integer_value=m_FastLWMA;

   parameters[4].type=TYPE_INT;
   parameters[4].integer_value=m_SlowLWMA;

   parameters[5].type=TYPE_INT;
   parameters[5].integer_value=m_IPC;

   parameters[6].type=TYPE_INT;
   parameters[6].integer_value=m_Digit;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,7,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(6)) return(false);

//---     Sidus
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CSidusSignal::LongCondition()
  {
//---       0  Sidus
   double Signal=m_indicator.GetData(0,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }
      else
        {
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }

//---      
   if(!m_SellPosClose) return(NO_SIGNAL);

   int Bars_=Bars(m_symbol.Name(),m_Ind_Timeframe);

   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_SHORT);
     }

//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CSidusSignal::ShortCondition()
  {
//---       1  Sidus
   double Signal=m_indicator.GetData(1,m_SignalBar);

//---   
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }
      else
        {
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }

//---      
   if(!m_BuyPosClose) return(NO_SIGNAL);

   int Bars_=Bars(Symbol(),m_Ind_Timeframe);
   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_LONG);
     }

//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
